/*
|------------------------------------------------------------------------------------------
|------------------------------------------------------------------------------------------
| File Name			  : MicroBolt_I2C_Functions.c																					
| Author			    : Micromint, Inc.						
| Copyright       : Copyright  2005, Micromint, Inc.																	
| Creation Date		: 12/30/05																							
| Version			    : 1.00																																												
| Spaces per tab	:	2																									
| Description		  : Main C file																																									 
| Revision			  :	Initial																																																						
|------------------------------------------------------------------------------------------
|------------------------------------------------------------------------------------------
*/

/*
|----------------------------------------------------
|     Includes
|----------------------------------------------------
*/

#include <ARM/philips/lpc210x.h>
#include <arm_macros.h>
#include "MicroBolt_I2C_EEPROM.h"
#include "MicroBolt_I2C_Functions.h"

unsigned char I2cSlaveAddress = 0;        // I2C slave address
unsigned char I2cBuffer[20];              // I2C application buffer
unsigned char I2cBufferIndex = 0;         // Index for I2C buffer
unsigned char I2cPacketDataSize = 0;      // Number of data bytes for an I2C packet
unsigned char I2cPacketType = 0;          // Read or Write packet type
unsigned char I2cPacketInProgress = 0;	  // I2C Packet in progress flag
unsigned char I2cSlaveStatusAckPoll = 0;  // I2C device status if ACK Write polling is supported			

/*
|------------------------------------------------------------------------------------------
|     Function        :   I2c_ISR
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   I2C interrupt and command processing
|     Author          :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/

#pragma interrupt_handler I2c_ISR

void I2c_ISR(void)
{
	switch(I2C_I2STAT)
			{
//****************************************************
// I2C Master Transmit Service Routines
//****************************************************
			case (0x08):																									// START condition transmitted
					{
					I2cBufferIndex = 0;																			  // Reset index counter
          I2C_I2DAT = I2cSlaveAddress | I2cPacketType;              // Desired slave address into I2C register OR'd with Read or Write
      	  I2C_I2CONCLR = I2C_STOP_BIT;	                            // Clear stop bit per datasheet
					break;
					}
//------------------------------------
			case (0x10):																									// Repeated START condition transmitted
					{
					I2cBufferIndex = 0;																			  // Reset index counter
          I2C_I2DAT = I2cSlaveAddress | I2cPacketType;              // Desired slave address into I2C register OR'd with Read or Write
      	  I2C_I2CONCLR = I2C_STOP_BIT;	                            // Clear stop bit per datasheet
					break;
					}
//------------------------------------
			case (0x18):																									// Slave address with WRITE transmitted and ACK received
					{
          if(I2cSlaveStatusAckPoll == ACK_POLL_IN_PROGRESS)         // Is this an ACK poll?
              {
              I2cSlaveStatusAckPoll = I2C_DEVICE_READY;             // Got an ACK for ACK Polling, device now ready
              I2cStop();                                            // Done ACK polling, send a stop condition to end packet         
              }
          else
              {
              I2C_I2DAT = I2cBuffer[I2cBufferIndex];                // Put data from I2c buffer into I2C register for sending 
              I2cBufferIndex++;                                     // Increment the buffer index
      	      I2C_I2CONCLR = I2C_START_BIT | I2C_STOP_BIT;		      // Clear start and stop bits per datasheet
              }        
  				break;
					}
//------------------------------------
			case (0x28):																									// Data byte transmitted and ACK received
					{
          I2C_I2DAT = I2cBuffer[I2cBufferIndex];                    // Put data from I2c buffer into I2C register for sending  
          I2cBufferIndex++;                                         // Increment the buffer index
      	  I2C_I2CONCLR = I2C_START_BIT | I2C_STOP_BIT;		          // Clear start and stop bits per datasheet  

          if (I2cBufferIndex > I2cPacketDataSize)                   // All the bytes from the I2C buffer sent out?
      	      {
              I2C_I2CONCLR = I2C_START_BIT;                         // Yes, Clear start bit per datasheet  
              I2cStop();                                            // Done sending data, send a stop condition to end packet
              }
					break;
					}
//------------------------------------
			case (0x20):																									// Slave address transmitted and NAK received
					{
          I2C_I2CONCLR = I2C_START_BIT;                             // Clear start bit per datasheet  
          I2cStop();                                                // Done sending data, send a stop condition to end packet
					break;
					}
//------------------------------------
			case (0x30):																									// Data byte transmitted and NAK received
					{
          I2C_I2CONCLR = I2C_START_BIT;                             // Clear start bit per datasheet  
          I2cStop();                                                // Done sending data, send a stop condition to end packet
					break;
					}
//------------------------------------
			case (0x38):																									// Master lost arbitration 
					{                                                         // Could send another START if desirable
      	  I2C_I2CONCLR = I2C_START_BIT | I2C_STOP_BIT;		          // Clear start and stop bits per datasheet 
					break;
					}
//------------------------------------

//****************************************************
// I2C Master Receive Service Routines
//****************************************************
			case (0x40):																									// Slave address with READ transmitted and ACK received
					{
					if (I2cPacketDataSize <= 1)		                            // Only one byte for the packet?
					    {
	            I2C_I2CONCLR = I2C_AA_BIT;                            // Send a NAK, don't want any more bytes
							I2cBufferIndex = 0;													          // Reset count index
							}
					else
							{
	            I2C_I2CONSET |= I2C_AA_BIT;                           // Send an ACK
						  }

      	  I2C_I2CONCLR = I2C_START_BIT | I2C_STOP_BIT;		          // Clear start and stop bits per datasheet            
					break;
					}
//------------------------------------
			case (0x50):																									// Data byte received and ACK returned
					{                                                         
					I2cBuffer[I2cBufferIndex] = I2C_I2DAT;									  // Grab the I2C data and put it into the I2C buffer
          I2cBufferIndex++;																          // Increment buffer count index
					if (I2cBufferIndex < (I2cPacketDataSize - 1))             // Still more data to come for packet?
					    {
	            I2C_I2CONSET |= I2C_AA_BIT;                           // Yes, Send an ACK
							}
					else
							{
	            I2C_I2CONCLR = I2C_AA_BIT;                            // No, Send a NAK, don't want any more bytes
							I2cBufferIndex = 0;													          // Reset count index
						  }

      	  I2C_I2CONCLR = I2C_START_BIT | I2C_STOP_BIT;		          // Clear start and stop bits per datasheet 
					break;
					}
//------------------------------------
			case (0x48):																									// Slave address with READ transmitted and NAK received
					{
          I2C_I2CONCLR = I2C_START_BIT;                             // Clear start bit per datasheet  
          I2cStop();                                                // Send a stop condition to end packet
					break;
					}
//------------------------------------
			case (0x58):																									// Data byte received and NAK returned
					{           
					I2cBuffer[I2cBufferIndex] = I2C_I2DAT;									  // Grab the I2C data and put it into the I2C buffer                                              
          I2C_I2CONCLR = I2C_START_BIT;                             // Clear start bit per datasheet  
          I2cStop();                                                // Send a stop condition to end packet
					break;
					}
//------------------------------------
			default:
					{
      	  I2C_I2CONCLR = I2C_START_BIT | I2C_STOP_BIT;		          // Clear start and stop bits per datasheet 
					break;
					}
			}
//------------------------------------

	I2C_I2CONCLR = I2C_SI_BIT;																		    // Clear I2C SI interrupt flag 
	VICVectAddr = VIC_ACK; 																						// Acknowledge Interrupt
}

/*
|------------------------------------------------------------------------------------------
|     Function        :   I2cStart
|     Inputs          :   I2cReadOrWriteNot
|     Outputs         :   None
|     Purpose         :   Send a start condition and setup packet type (Read or Write)
|     Author			    :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/

void I2cStart(unsigned char RequestedI2cPacketType)
  {
  I2cPacketInProgress = TRUE;                   // Flag for stating an I2C packet is about to happen
  I2cPacketType = RequestedI2cPacketType;       // Flag for stating what kind of packet it is; Read, Write, or ACK poll
	I2C_I2CONSET |= I2C_START_BIT;                // Set the start bit 
  }

/*
|------------------------------------------------------------------------------------------
|     Function        :   I2cStop
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   Send a stop condition
|     Author			    :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/

void I2cStop(void)
  {
	I2C_I2CONSET |= I2C_STOP_BIT;                 // Set the stop bit 
  I2cPacketInProgress = FALSE;                  // Flag for stating I2C packet is done
  }

/*
|------------------------------------------------------------------------------------------
|     Function        :   I2cSlaveAckPoll
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   Poll the currently addressed I2C slave and wait for an ACK
|     Author			    :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/

void I2cSlaveAckPoll(void)
  {
  I2cSlaveStatusAckPoll = ACK_POLL_IN_PROGRESS;         // Status set to busy after writing to EEPROM
  I2cPacketDataSize = 0;                                // No data for an ACK poll, just the current slave address

  while(I2cSlaveStatusAckPoll == ACK_POLL_IN_PROGRESS)  // Slave still busy?
      {
      I2cStart(I2C_WRITE);                              // Send out an I2C Start condition Write for an ACK poll
      while(I2cPacketInProgress == TRUE);               // Wait here for I2C packet to complete 
      }
  }